import matplotlib.pyplot as plt
import math
import random
import os

# Lecture des données
def read_data(file='ch.csv'):
    cities = []
    coordinates = {}
    # Utilisation de l'encodage utf-8 pour lire le fichier, la première ligne contient les noms des colonnes, elle est ignorée
    for line in open(file, encoding="utf-8").readlines()[1:]:
        city, lat, lng = line.split(',')[0:3]
        lat = float(lat)
        lng = float(lng)
        cities.append(city)
        coordinates[city] = (lat, lng)
    return cities, coordinates

# Calcul de la distance entre deux points donnés par leur latitude et longitude
def haversine(coord1, coord2, R=6371):
    lat1, lon1 = coord1
    lat2, lon2 = coord2
    dlat = math.radians(lat2 - lat1)
    dlon = math.radians(lon2 - lon1)
    a = math.sin(dlat / 2) ** 2 + math.cos(math.radians(lat1)) * math.cos(math.radians(lat2)) * math.sin(dlon / 2) ** 2
    c = 2 * math.asin(math.sqrt(a))
    return R * c

# Création de la matrice d'adjacence
def create_adj_matrix(cities, coordinates):
    # Initialisation de la matrice d'adjacence
    adj = [[0 for _ in range(len(cities))] for _ in range(len(cities))]
    # Remplissage de la matrice d'adjacence
    for i in range(len(cities)):
        for j in range(len(cities)):
            adj[i][j] = haversine(coordinates[cities[i]], coordinates[cities[j]])
    return adj

# Calcul du coût d'une solution
def cost(solution):
    c = 0
    for i in range(len(solution)):
        city1 = solution[i]
        city2 = solution[(i + 1) % len(solution)] # Permet de revenir à la première ville après la dernière
        idx_1 = cities.index(city1)
        idx_2 = cities.index(city2)

        w = adj[idx_1][idx_2]
        c += w
    return int(c)

# Exemple d'utilisation
for f in ['ch.csv', 'world.csv']:

    print('Fichier', f)

    # Exemple d'utilisation
    cities, coordinates = read_data(f)
    adj = create_adj_matrix(cities, coordinates)

    # Création d'une solution aléatoire
    solution = cities.copy()
    random.shuffle(solution)

    # Visualisation de la solution
    for i in range(len(solution)):
        city1 = solution[i]
        city2 = solution[(i + 1)%len(solution)]

        idx_1 = cities.index(city1)
        idx_2 = cities.index(city2)
        plt.plot([coordinates[city1][1], coordinates[city2][1]], [coordinates[city1][0], coordinates[city2][0]],
                 color='black')

    plt.title(f + '\n Coût du cycle Hamiltonien : ' + str(cost(solution)))
    plt.show()
    plt.clf()
    print('    Aléatoire :', cost(solution))